#! /usr/bin/env node
require("babel-register");
// Required Libraries
const parseArgs = require("minimist");
const _ = require("lodash");
const nodemon = require("nodemon");
const path = require("path");
const fs = require("fs");
const webpack = require("webpack");

const currentDir = process.env.PWD;
process.env.__p_root = currentDir + "/";
process.env.__c_root = __dirname;
const args = parseArgs(process.argv.slice(2));
const allCommands = args["_"];

let userCommand = "start";

if (allCommands.length) {
	userCommand = _.first(allCommands);
}

const NODE_ENV = process.env.NODE_ENV || "development";

// Set NODE_ENV if not already set
process.env.NODE_ENV = NODE_ENV;

// Disable babel cache
process.env.BABEL_DISABLE_CACHE = 1;

let allExecutablePaths = process.env.PATH.split(path.delimiter);
allExecutablePaths.unshift(__dirname);
allExecutablePaths.unshift(`${__dirname}${path.sep}.bin`);
allExecutablePaths.unshift(`${__dirname}${path.sep}node_modules${path.sep}.bin`);

function findCommandPath(com) {
	let execPath = "";
	_.each(allExecutablePaths, executablePath => {
		if (execPath.length) return;

		if (fs.existsSync(`${executablePath}${path.sep}${com}`)) {
			return execPath = `${executablePath}${path.sep}${com}`;
		}
		if (fs.existsSync(`${executablePath}${path.sep}${com}.exe`)) {
			return execPath = `${executablePath}${path.sep}${com}.exe`;
		}
		if (fs.existsSync(`${executablePath}${path.sep}${com}.cmd`)) {
			return execPath = `${executablePath}${path.sep}${com}.cmd`;
		}
	});
	return execPath.length ? execPath: undefined;
}

switch(userCommand) {
	case "start": {
		nodemon({
			script: `${currentDir}/src/server.js`,
			execMap: {
				"js": findCommandPath("babel-node")
			},
			"watch": [
				`${currentDir}/src/server.js`
			]
		});
		break;
	}
	case "build": {
		process.env.NODE_ENV = "production";
		const prodClientConfig = require(path.resolve(path.join(__dirname, "webpack", "prod.client.babel.js"))).default;
		webpack(prodClientConfig, (clientErr, clientStats) => {

			if (!clientErr)  {
				const prodServerConfig = require(path.resolve(path.join(__dirname, "webpack", "prod.server.babel.js"))).default;
				webpack(prodServerConfig, (serverErr, serverStats) => {
					if (serverErr) {
						// eslint-disable-next-line
						console.log(serverErr);
					}
				});
			} else {
				// eslint-disable-next-line
				console.log(clientErr);
			}
		})
		break;
	}
}

 //nodemon --exec babel-node --watch ../src/server.js --watch ./src --watch ./webpack ../src/server.js
